#include <iostream>
#include <string>
#include <new>
using namespace std;

struct abonent
{
	string imie;
	string nazwisko;
	string numer;

	abonent *lewy;
	abonent *prawy;
};

// prototypy
bool dodajAbonenta(abonent **korzen, abonent *dane);
abonent *szukajAbonenta(abonent *korzen, string nazwisko, unsigned long *kroki = NULL);
void usunDrzewo(abonent **korzen);
void wyswietlDrzewo(abonent *korzen);
abonent *wczytajAbonenta();
void wypiszAbonenta(abonent *co);

int main()
{
	cout << "Program utworzy baze numerow telefonicznych" << endl << endl;	
	cout << "Podaj czynnosc do wykonania:" << endl;
	cout << "d - dodaj wpis do bazy" << endl;
	cout << "s - szukaj osoby o podanym nazwisku" << endl;
	cout << "w - wypisz liste rekordow w bazie" << endl;
	cout << "z - zakoncz dzialanie programu" << endl << endl;

	abonent *drzewo = NULL;
	unsigned long licznik = 0;
	char o;
	do {
		cout << "Podaj czynnosc do wykonania [d/s/w/z]: ";
		cin >> o; cout << endl;
		cin.ignore();

		// Zamie na du liter
		o = toupper(o);		
		switch(o)
		{
			case 'D': {
					cout << "Wpisz dane nowej osoby: " << endl <<endl;
					abonent *nowy = wczytajAbonenta();
					if(nowy != NULL) {
						if(dodajAbonenta(&drzewo, nowy)) licznik++;
						else cout << "Blad (konflikt nazwisk)" << endl;
					}
					else cout << "Blad utworzenia nowego abonenta" << endl;
					break;
				}
				
			case 'S': {
					cout << "Podaj szukane nazwisko: ";
					string szukany;
					getline(cin, szukany);
					
					unsigned long ile;
					abonent *znaleziony = szukajAbonenta(drzewo, szukany, &ile);
					if(znaleziony == NULL)
						cout << "Abonenta nie ma w bazie, przeglad objal " << ile << " z " 
							<< licznik  << " rekordow" << endl << endl;
					else {
						cout << "Znalezienie abonenta wymagalo przegladu " << ile << " z " 
							<< licznik << " rekordow bazy" << endl << endl;
						cout << "Znaleziono nastepujaca osobe:" << endl;
						wypiszAbonenta(znaleziony);
						cout << endl;
					}
					break;
				}
				
			case 'W':
				cout << "Baza zawiera obecnie " << licznik << " abonentow:" << endl << endl;
				wyswietlDrzewo(drzewo);
				cout << endl;
				break;
				
			case 'Z': break;
			default: cout << "Dopuszczalne opcje to d [dodaj], s [szukaj], w [wypisz] lub z [zakoncz]" << endl;
		}
	} while(o != 'Z');
	usunDrzewo(&drzewo);
}

bool dodajAbonenta(abonent **korzen, abonent *dane)
{
	if(korzen == NULL || dane == NULL) return false;
	if(*korzen == NULL) {
		// W wypadku pustego drzewa dodajemy nowy element jako korzen
		*korzen = dane;
		return true;
	} else {
		abonent *aktualny = *korzen;
		while(aktualny != NULL)
		{
			if(aktualny->nazwisko == dane->nazwisko) return false;
			if(dane->nazwisko < aktualny->nazwisko)
				if(aktualny->lewy != NULL) aktualny = aktualny->lewy;
				else {
					aktualny->lewy = dane;
					return true;
				}
			else // dane->nazwisko > aktualny->nazwisko
				if(aktualny->prawy != NULL) aktualny = aktualny->prawy;
				else {
					aktualny->prawy = dane;
					return true;
				}
		}
		return false;
	}
}

abonent *szukajAbonenta(abonent *korzen, string nazwisko, unsigned long *kroki)
{
	unsigned long i = 0;
	abonent *aktualny = korzen;
	while(aktualny != NULL) {
		i++;
		if(aktualny->nazwisko == nazwisko) {
			if(kroki != NULL) *kroki = i;
			return aktualny;
		}
		if(nazwisko < aktualny->nazwisko) aktualny = aktualny->lewy;
			else aktualny = aktualny->prawy;
	}
	if(kroki != NULL) *kroki = i;
	return NULL;
}

void wyswietlDrzewo(abonent *korzen)
{
	if(korzen == NULL) cout << "Puste drzewo wyszukiwania" << endl;
	else {
		if(korzen->lewy != NULL) wyswietlDrzewo(korzen->lewy);
		wypiszAbonenta(korzen); cout << endl;
		if(korzen->prawy != NULL) wyswietlDrzewo(korzen->prawy);
	}
}

void usunDrzewo(abonent **korzen)
{
	if(korzen != NULL && *korzen != NULL) {
		// rekurencyjnie usun oba poddrzewa
		usunDrzewo( &(*korzen)->lewy );
		usunDrzewo( &(*korzen)->prawy );
		// Usun aktualny wierzcholek
		delete *korzen;
		*korzen = NULL;
	}
}

abonent *wczytajAbonenta()
{
	abonent *nowy;
	try {
		nowy = new abonent;
	}
	catch(bad_alloc) {
		return NULL;
	}
	
	cout << "Podaj imie: ";
	getline(cin, nowy->imie);
	cout << "Podaj nazwisko: ";
	getline(cin, nowy->nazwisko);
	cout << "Podaj numer: ";
	getline(cin, nowy->numer);
	nowy->lewy = nowy->prawy = NULL;
	return nowy;
}

void wypiszAbonenta(abonent *co)
{
	if(co != NULL) {
		cout << "imie: " << co->imie << endl;
		cout << "nazwisko: " << co->nazwisko << endl;
		cout << "numer: " << co->numer << endl;
	}
}
